@@ -44,13 +44,6 @@
 #include <stdlib.h>
 #include <ctype.h>
 
-#ifdef G_OS_WIN32
-/* No hardcoded paths in the binary, thanks */
-#undef PKGLIBDIR
-/* It's OK to leak this, as PKGLIBDIR is invoked only once */
-#define PKGLIBDIR g_strconcat (g_win32_get_package_installation_directory (PACKAGE, NULL), "\\lib\\pkgconfig", NULL)
-#endif
-
 static void verify_package (Package *pkg);
 
 static GHashTable *packages = NULL;
@@ -61,6 +54,7 @@ static GSList *search_dirs = NULL;
 static int scanned_dir_count = 0;
 
 gboolean disable_uninstalled = FALSE;
+gboolean ignore_requires = FALSE;
 
 void
 add_search_dir (const char *path)
@@ -68,6 +62,27 @@ add_search_dir (const char *path)
   search_dirs = g_slist_append (search_dirs, g_strdup (path));
 }
 
+void
+add_search_dirs (const char *path, const char *separator)
+{
+      char **search_dirs;
+      char **iter;
+
+      search_dirs = g_strsplit (path, separator, -1);
+    
+      iter = search_dirs;
+      while (*iter)
+        {
+          debug_spew ("Adding directory '%s' from PKG_CONFIG_PATH\n",
+                      *iter);
+          add_search_dir (*iter);
+          
+          ++iter;
+        }
+      
+      g_strfreev (search_dirs);
+}
+
 #ifdef G_OS_WIN32
 /* Guard against .pc file being installed with UPPER CASE name */
 # define FOLD(x) tolower(x)
@@ -183,15 +198,30 @@ scan_dir (const char *dirname)
     }
 }
 
+static Package *
+add_virtual_pkgconfig_package (void)
+{
+  Package *pkg = NULL;
+
+  pkg = g_new0 (Package, 1);
+
+  pkg->key = g_strdup ("pkg-config");
+  pkg->version = g_strdup (VERSION);
+  pkg->name = g_strdup ("pkg-config");
+  pkg->description = g_strdup ("pkg-config is a system for managing "
+			       "compile/link flags for libraries");
+  pkg->url = g_strdup ("http://www.freedesktop.org/software/pkgconfig/");
+
+  debug_spew ("Adding virtual 'pkg-config' package to list of known packages\n");
+  g_hash_table_insert (packages, pkg->key, pkg);
+
+  return pkg;
+}
+
 void
 package_init ()
 {
   static gboolean initted = FALSE;
-  const char *pkglibdir;
-
-  pkglibdir = g_getenv ("PKG_CONFIG_LIBDIR");
-  if (pkglibdir == NULL)
-    pkglibdir = PKGLIBDIR;
 
   if (!initted)
     {
@@ -201,8 +231,9 @@ package_init ()
       locations = g_hash_table_new (g_str_hash, g_str_equal);
       path_positions = g_hash_table_new (g_str_hash, g_str_equal);
       
+      add_virtual_pkgconfig_package ();
+
       g_slist_foreach (search_dirs, (GFunc)scan_dir, NULL);
-      scan_dir (pkglibdir);
     }
 }
 
@@ -289,7 +320,7 @@ internal_get_package (const char *name, gboolean warn, gboolean check_compat)
     }
 
   debug_spew ("Reading '%s' from file '%s'\n", name, location);
-  pkg = parse_package_file (location);
+  pkg = parse_package_file (location, ignore_requires);
   
   if (pkg == NULL)
     {
@@ -652,9 +683,7 @@ add_env_variable_to_list (GSList *list, const gchar *env)
   gchar **values;
   gint i;
 
-  /* FIXME: the separator should be a ';' on Windows
-   */
-  values = g_strsplit (env, ":", 0);
+  values = g_strsplit (env, G_SEARCHPATH_SEPARATOR_S, 0);
   for (i = 0; values[i] != NULL; i++)
     {
       list = g_slist_append (list, g_strdup (values[i]));
@@ -696,14 +725,14 @@ verify_package (Package *pkg)
   if (pkg->version == NULL)
     {
       verbose_error ("Package '%s' has no Version: field\n",
-                     pkg->name);
+                     pkg->key);
       exit (1);
     }
 
   if (pkg->description == NULL)
     {
       verbose_error ("Package '%s' has no Description: field\n",
-                     pkg->description);
+                     pkg->key);
       exit (1);
     }
   
@@ -730,6 +759,9 @@ verify_package (Package *pkg)
                              ver->version,
                              req->name,
                              req->version);
+              if (req->url)
+                verbose_error ("You may find new versions of %s at %s\n",
+                               req->name, req->url);
 
               exit (1);
             }
@@ -784,8 +816,10 @@ verify_package (Package *pkg)
   /* We make a list of system directories that gcc expects so we can remove
    * them.
    */
+#ifndef G_OS_WIN32
   system_directories = g_slist_append (NULL, g_strdup ("/usr/include"));
-  
+#endif
+
   c_include_path = g_getenv ("C_INCLUDE_PATH");
   if (c_include_path != NULL)
     {
@@ -848,25 +882,31 @@ verify_package (Package *pkg)
   g_slist_foreach (system_directories, (GFunc) g_free, NULL);
   g_slist_free (system_directories);
 
+#ifdef PREFER_LIB64
+#define SYSTEM_LIBDIR "/usr/lib64"
+#else
+#define SYSTEM_LIBDIR "/usr/lib"
+#endif
   count = 0;
   iter = pkg->L_libs;
   while (iter != NULL)
     {
-      if (strcmp (iter->data, "-L/usr/lib") == 0 ||
-          strcmp (iter->data, "-L /usr/lib") == 0)
+      if (strcmp (iter->data, "-L" SYSTEM_LIBDIR) == 0 ||
+          strcmp (iter->data, "-L " SYSTEM_LIBDIR) == 0)
         {
-          debug_spew ("Package %s has -L/usr/lib in Libs\n",
+          debug_spew ("Package %s has -L" SYSTEM_LIBDIR " in Libs\n",
                       pkg->name);
           if (g_getenv ("PKG_CONFIG_ALLOW_SYSTEM_LIBS") == NULL)
             {              
               iter->data = NULL;
               ++count;
-              debug_spew ("Removing -L/usr/lib from libs for %s\n", pkg->key);
+              debug_spew ("Removing -L" SYSTEM_LIBDIR " from libs for %s\n", pkg->key);
             }
         }
 
       iter = iter->next;
     }
+#undef SYSTEM_LIBDIR
 
   while (count)
     {
@@ -1405,6 +1445,8 @@ print_package_list (void)
 {
   int mlen = 0;
   
+  ignore_requires = TRUE;
+
   g_hash_table_foreach (locations, max_len_foreach, &mlen);
   g_hash_table_foreach (locations, packages_foreach, GINT_TO_POINTER (mlen + 1));
 }
